Stan Math Library: stan/math/prim/scal/prob/double_exponential_ccdf_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

double_exponential_ccdf_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_DOUBLE_EXPONENTIAL_CCDF_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_DOUBLE_EXPONENTIAL_CCDF_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_finite.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
11 #include <stan/math/prim/scal/fun/value_of.hpp>
12 #include <stan/math/prim/scal/fun/log1m.hpp>
13 #include <stan/math/prim/scal/fun/constants.hpp>
14 #include <stan/math/prim/scal/meta/include_summand.hpp>
15 #include <stan/math/prim/scal/fun/sign.hpp>
16 #include <boost/random/uniform_01.hpp>
17 #include <boost/random/variate_generator.hpp>
18 #include <cmath>
19
20 namespace stan {
21
22 namespace math {
23
24 template <typename T_y, typename T_loc, typename T_scale>
25 typename return_type<T_y, T_loc, T_scale>::type
26 double_exponential_ccdf_log(const T_y& y, const T_loc& mu,
27 const T_scale& sigma) {
28 static const char* function("stan::math::double_exponential_ccdf_log");
29 typedef typename stan::partials_return_type<T_y, T_loc, T_scale>::type
30 T_partials_return;
31
32 using stan::math::check_finite;
33 using stan::math::check_not_nan;
34 using stan::math::check_positive_finite;
35 using stan::math::check_consistent_sizes;
36 using stan::math::value_of;
37
38 T_partials_return ccdf_log(0.0);
39
40 // check if any vectors are zero length
41 if (!(stan::length(y)
42 && stan::length(mu)
43 && stan::length(sigma)))
44 return ccdf_log;
45
46 check_not_nan(function, "Random variable", y);
47 check_finite(function, "Location parameter", mu);
48 check_positive_finite(function, "Scale parameter", sigma);
49 check_consistent_sizes(function,
50 "Random variable", y,
51 "Location parameter", mu,
52 "Scale Parameter", sigma);
53
54 using std::log;
55 using std::exp;
56 using stan::math::log1m;
57 using std::exp;
58
59 OperandsAndPartials<T_y, T_loc, T_scale>
60 operands_and_partials(y, mu, sigma);
61
62 VectorView<const T_y> y_vec(y);
63 VectorView<const T_loc> mu_vec(mu);
64 VectorView<const T_scale> sigma_vec(sigma);
65 const double log_half = std::log(0.5);
66 size_t N = max_size(y, mu, sigma);
67
68 for (size_t n = 0; n < N; n++) {
69 const T_partials_return y_dbl = value_of(y_vec[n]);
70 const T_partials_return mu_dbl = value_of(mu_vec[n]);
71 const T_partials_return sigma_dbl = value_of(sigma_vec[n]);
72 const T_partials_return scaled_diff = (y_dbl - mu_dbl) / sigma_dbl;
73 const T_partials_return inv_sigma = 1.0 / sigma_dbl;
74 if (y_dbl < mu_dbl) {
75 // log ccdf
76 ccdf_log += log1m(0.5 * exp(scaled_diff));
77
78 // gradients
79 const T_partials_return rep_deriv = 1.0
80 / (2.0 * exp(-scaled_diff) - 1.0);
81 if (!is_constant_struct<T_y>::value)
82 operands_and_partials.d_x1[n] -= rep_deriv * inv_sigma;
83 if (!is_constant_struct<T_loc>::value)
84 operands_and_partials.d_x2[n] += rep_deriv * inv_sigma;
85 if (!is_constant_struct<T_scale>::value)
86 operands_and_partials.d_x3[n] += rep_deriv * scaled_diff
87 * inv_sigma;
88 } else {
89 // log ccdf
90 ccdf_log += log_half - scaled_diff;
91
92 // gradients
93 if (!is_constant_struct<T_y>::value)
94 operands_and_partials.d_x1[n] -= inv_sigma;
95 if (!is_constant_struct<T_loc>::value)
96 operands_and_partials.d_x2[n] += inv_sigma;
97 if (!is_constant_struct<T_scale>::value)
98 operands_and_partials.d_x3[n] += scaled_diff * inv_sigma;
99 }
100 }
101 return operands_and_partials.value(ccdf_log);
102 }
103 }
104 }
105 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
include_summand.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::OperandsAndPartials::d_x3
VectorView< T_return_type, false, true > d_x3
Definition: OperandsAndPartials.hpp:41
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
log1m.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
constants.hpp
sign.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
check_finite.hpp
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::double_exponential_ccdf_log
return_type< T_y, T_loc, T_scale >::type double_exponential_ccdf_log(const T_y &y, const T_loc &mu, const T_scale &sigma)
Definition: double_exponential_ccdf_log.hpp:26
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::log1m
fvar< T > log1m(const fvar< T > &x)
Definition: log1m.hpp:16
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39

[ Stan Home Page ] © 2011–2016, Stan Development Team.